from django.apps import AppConfig


class IndexConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_ai_core.contrib.index"
    verbose_name = "Django AI Core Vector Indexing Module"
