from django.apps import AppConfig


class CoreConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_ai_core"
    label = "django_ai_core"
    verbose_name = "Django AI Core"
