# Generated by Django 5.2.3 on 2025-09-24 15:26

import pgvector.django.vector
from django.db import migrations, models
from pgvector.django import VectorExtension


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        VectorExtension(),
        migrations.CreateModel(
            name="PgVectorEmbedding",
            fields=[
                ("index_name", models.CharField(max_length=255)),
                (
                    "document_key",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("content", models.TextField()),
                ("metadata", models.JSONField(default=dict)),
                ("vector", pgvector.django.vector.VectorField()),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
